/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {setZoomedExerciseRelease} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/zoomedRelease/zoomedReleaseDialogActions";
import {
  deleteExerciseReleaseAsyncAndReload, setActivePaginationPage,
  setIsZoomedExerciseReleaseDisplayed, setPaginationPageSize,
  setSortByKey
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {openEditExerciseRelease} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/changeExerciseReleaseDialog/changeExerciseReleaseDialogActions";
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import {leftIconPopupOffset} from '../../../../constants'
import {copyToClipboard} from "../../../../helpers/clipboard";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../../helpers/tablesHelper";
import {HelpPopup} from "../../../helpers/helpPopup";
import {askDialog} from "../../../../helpers/dialogHelper";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {MyPopup} from "../../../helpers/myPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../../helpers/paginationTableFooter'
import {ReleaseDurationType} from '../../../../types/ReleaseDurationType'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releases: rootState.releaseSiteState.releases,
    exercises: rootState.releaseSiteState.exercise,
    comesFromOwnExercisesSite: rootState.releaseSiteState.comesFromOwnExercisesSite,
    selectedExerciseId: rootState.releaseSiteState.selectedExerciseId,

    sortByKey: rootState.releaseSiteState.pagination.orderByProp,
    sortDirection: rootState.releaseSiteState.pagination.isOrderByDesc,
    pagination: rootState.releaseSiteState.pagination,

    groupRole: rootState.releaseSiteState.groupRole,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setZoomedExerciseRelease,
                                                                        setIsZoomedExerciseReleaseDisplayed,
                                                                        openEditExerciseRelease,

                                                                        deleteExerciseReleaseAsyncAndReload,
                                                                        setSortByKey,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class ListView extends React.Component<Props, any> {
  render(): JSX.Element {
    const list: ReadonlyArray<ExerciseReleaseFromBackend> = this.props.releases


    return (
      <div>

        {
          this.props.exercises && this.props.exercises.isOnlyVisibleToMe &&
          <div style={{
            marginBottom: '1em',
            marginLeft: '1em'
          }}>
            <p>
              <Icon name="exclamation triangle"/>
              {
                getI18n(this.props.langId,
                        "The exercise is only visible to the owner and thus all releases for this exercise will only work for the owner"
                )
              }
            </p>
          </div>
        }

        <Table selectable sortable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExerciseReleaseFromBackend>(this.props.sortByKey, 'isReleased',
                                                                              this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isReleased',
                                                rotateSorting<ExerciseReleaseFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'isReleased'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Is activated')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Release code')
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Start at')
                  }
                </span>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                'If the start type is automatic then the automatic start time is changed to the actual start time.'
                )}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'End at')
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  //this somehow cannot properly sorted if we have no working time (unlimited) this is the default value...
                }
                <span>
                  {
                    getI18n(this.props.langId, 'Working time in min')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExerciseReleaseFromBackend>(this.props.sortByKey,
                                                                              'isVisibleToAllAfterRelease',
                                                                              this.props.sortDirection
                  )
                }
              >
                <div className="inline">
                  <span className="clickable mar-right-half"
                        onClick={() => {
                          this.props.setSortByKey('isVisibleToAllAfterRelease',
                                                  rotateSorting<ExerciseReleaseFromBackend>(this.props.sortDirection,
                                                                                            this.props.sortByKey,
                                                                                            'isVisibleToAllAfterRelease'
                                                  )
                          )
                        }}
                  >
                    {
                      getI18n(this.props.langId, 'Is visible for all')
                    }
                  </span>
                  <HelpPopup
                    defaultText={getI18n(this.props.langId,
                                         'If the release is visible for all then every user can see the release without entering the code. If the exercise \'is only visible to owner\' then the release cannot be visible for all and only the owner can enter the release code'
                    )}
                    wide/>
                </div>
              </Table.HeaderCell>

              <Table.HeaderCell>

                <span>
                  {
                    getI18n(this.props.langId, 'Miscellaneous')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExerciseReleaseFromBackend>(this.props.sortByKey,
                                                                              'note',
                                                                              this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('note',
                                                rotateSorting<ExerciseReleaseFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'note'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Note')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExerciseReleaseFromBackend>(this.props.sortByKey,
                                                                              'solutionsCount',
                                                                              this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('solutionsCount',
                                                rotateSorting<ExerciseReleaseFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'solutionsCount'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Submissions')
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Change')
                }
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Delete')
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exerciseRelease, index) => {


                // tslint:disable-next-line
                let editNode = null

                if (this.props.exercises !== null && this.props.exercises.isPermanentlyLocked) {

                  editNode = <HelpPopup defaultText={getI18n(this.props.langId, "Exercise is lock")}/>

                } else if (((this.props.groupRole && this.props.groupRole.canManageExerciseReleases) || this.props.comesFromOwnExercisesSite) &&
                  (exerciseRelease.hasAutomaticAssessmentStarted === false || exerciseRelease.hasAutomaticAssessmentFinished) &&
                  (this.props.exercises !== null && this.props.exercises.isPermanentlyLocked === false)) {

                  editNode = (<div className="clickable"
                                   onClick={() => {
                                     this.props.openEditExerciseRelease(exerciseRelease, this.props.exercises)
                                   }}
                  >
                    <Icon name="write"/>
                  </div>)

                } else if (((this.props.groupRole && this.props.groupRole.canManageExerciseReleases) || this.props.comesFromOwnExercisesSite) &&
                  (exerciseRelease.hasAutomaticAssessmentStarted === true && exerciseRelease.hasAutomaticAssessmentFinished === false) &&
                  (this.props.exercises !== null && this.props.exercises.isPermanentlyLocked === false)) {

                  editNode = (<div>
                    <HelpPopup defaultText={getI18n(this.props.langId,
                                                    "This automatic assessment is running, thus the release cannot be edited currently. Refresh the site to check again"
                    )}/>
                  </div>)
                }

                // tslint:disable-next-line
                let deleteNode = null

                if (this.props.exercises !== null && this.props.exercises.isPermanentlyLocked) {

                  editNode = <HelpPopup defaultText={getI18n(this.props.langId, "Exercise is lock")}/>

                } else if ((this.props.groupRole && this.props.groupRole.canManageExerciseReleases) || this.props.comesFromOwnExercisesSite) {

                  deleteNode = (<div className="clickable"
                                     onClick={async () => {

                                       const shouldRemove = await askDialog(
                                         getI18n(this.props.langId, 'Delete release'),
                                         `${getI18n(this.props.langId,
                                                    'Are you sure you want to delete the release? This will also delete all submissions (including assessments)!'
                                         )} (${exerciseRelease.generatedCode})`,
                                         this.props.langId
                                       )

                                       if (shouldRemove) {
                                         this.props.deleteExerciseReleaseAsyncAndReload(exerciseRelease.id,
                                                                                        this.props.selectedExerciseId,
                                                                                        true,
                                                                                        this.props.comesFromOwnExercisesSite
                                         )
                                       }

                                     }}
                  >
                    <Icon name="trash"/>
                  </div>)
                }


                return (
                  <Table.Row key={exerciseRelease.id}>
                    <Table.Cell>

                      <Checkbox disabled checked={exerciseRelease.isReleased}/>

                    </Table.Cell>
                    <Table.Cell>
                      <div className="flexed">
                        <Icon name="zoom" className="clickable mar-right-half"
                              onClick={() => {
                                this.props.setZoomedExerciseRelease(exerciseRelease)
                                this.props.setIsZoomedExerciseReleaseDisplayed(true)
                              }}
                        />
                        <MyPopup
                          content={exerciseRelease.generatedCode}
                          trigger={
                            <Icon name="copy" className="clickable mar-right-half"
                                  onClick={() => {
                                    copyToClipboard(exerciseRelease.generatedCode)
                                  }}
                            />
                          }
                        />
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {
                        (exerciseRelease.releaseStartType === ReleaseStartType.Manual && exerciseRelease.releasedAt) &&
                        displayDateAndTime(exerciseRelease.releasedAt, -1)
                      }
                      {
                        exerciseRelease.releaseStartType === ReleaseStartType.Automatic
                        && exerciseRelease.automaticStartAt && exerciseRelease.releasedAt === null
                        &&
                        `${displayDateAndTime(exerciseRelease.automaticStartAt, -1)} (${getI18n(this.props.langId,
                                                                                                'Automatic'
                        )})`
                      }

                      {
                        //after release display the actual release time
                      }
                      {
                        exerciseRelease.releaseStartType === ReleaseStartType.Automatic
                        && exerciseRelease.automaticStartAt && exerciseRelease.releasedAt !== null
                        &&
                        `${displayDateAndTime(exerciseRelease.releasedAt, -1)} (${getI18n(this.props.langId,
                                                                                          'Automatic'
                        )})`
                      }
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        {
                          (exerciseRelease.releaseStartType === ReleaseStartType.Manual || exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely) &&
                          getI18n(this.props.langId, 'Manual')
                        }
                        {
                          (exerciseRelease.releaseStartType === ReleaseStartType.Automatic &&
                            exerciseRelease.releaseDurationType === ReleaseDurationType.Limited
                            && exerciseRelease.automaticEndAt) &&
                          `${displayDateAndTime(exerciseRelease.automaticEndAt, -1)} (${getI18n(this.props.langId,
                                                                                                'Automatic'
                          )})`
                        }
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        {
                          exerciseRelease.hasLimitedWorkingTime
                          ? `${exerciseRelease.availableWorkingTimeInMinutes.toString()} ${getI18n(this.props.langId,
                                                                                                   'm'
                          )}`
                          : <span>&infin;</span>
                        }
                      </div>
                    </Table.Cell>

                    <Table.Cell>

                      <Checkbox disabled checked={exerciseRelease.isVisibleToAllAfterRelease}/>

                    </Table.Cell>

                    <Table.Cell>

                      <div>

                        <MyPopup
                          trigger={<Icon name='unhide'
                                         className={exerciseRelease.hideInOverviews
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           'Hides the release from the users in all exercise overviews and statistic views'
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Hide release in overviews')}
                          wide
                        />

                        <MyPopup
                          trigger={<Icon name='sidebar'
                                         className={exerciseRelease.hideSiteHeaderBar
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           'Hides the header bar (checked) or displays it (unchecked). The site header bar is the topmost bar where the site menu, user menu is located.'
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Hide site header bar')}
                          wide
                        />

                        <MyPopup
                          trigger={<Icon name='clipboard'
                                         className={exerciseRelease.shouldClearClipboard
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           "Shows an overlay to resume when the window/tab lost focus. This will clear the clipboard when accepted else the user can't proceed."
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Clear clipboard on focus (EXPERIMENTAL, known issues)')}
                          wide
                        />

                        <MyPopup
                          trigger={<Icon name='print'
                                         className={exerciseRelease.hidePrintOptions
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           'This will hide all print options (e.g. print task description, code editor)'
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Hide print options')}
                          wide
                        />

                        <MyPopup
                          trigger={<Icon name='reply'
                                         className={exerciseRelease.hideExerciseLeaveActions
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           'This will hide all actions and buttons the user could use to leave the exercise (and e.g. select another exercise). The buttons in the site header bar are not affected (use the appropriate option for that).'
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Hide leave exercise actions')}
                          wide
                        />

                        <MyPopup
                          trigger={<Icon name='unordered list'
                                         className={exerciseRelease.showAdditionalLogButton
                                                    ? 'hoverable'
                                                    : 'hoverable-disabled'}/>}
                          content={getI18n(this.props.langId,
                                           'This will show an additional log button because some options might hide the user header bar thus one cannot show the log and e.g. download the log'
                          )}
                          offset={leftIconPopupOffset}
                          header={getI18n(this.props.langId, 'Show additional log-button')}
                          wide
                        />

                      </div>

                      {
                        /*
                         <Checkbox disabled checked={exerciseRelease.hideInOverviews}/>
                      <Checkbox disabled checked={exerciseRelease.shouldClearClipboard}/>
                         */
                      }

                    </Table.Cell>

                    <Table.Cell>
                      <span>{exerciseRelease.note}</span>
                    </Table.Cell>


                    <Table.Cell>
                      <div>
                        <Link to={constants.getSubmissionsSiteToLinkPath(this.props.selectedExerciseId,
                                                                         this.props.comesFromOwnExercisesSite,
                                                                         exerciseRelease.id
                        )}>
                          <Icon name="send"/>
                          <span>{exerciseRelease.solutionsCount}</span>
                        </Link>
                      </div>

                    </Table.Cell>

                    <Table.Cell>
                      {
                        editNode
                      }
                    </Table.Cell>


                    <Table.Cell>
                      {
                        deleteNode
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setActivePaginationPage={this.props.setActivePaginationPage}
            setPaginationPageSize={this.props.setPaginationPageSize}
          />

        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ListView)
