import swal, {SweetAlertOptions, SweetAlertType} from 'sweetalert2'
import Logger from "./logger";
import {getI18n, KnownLangs} from "../../i18n/i18nRoot";


export enum DialogType {
  success,
  warning,
  info,
  question,
  error,
}


function translateDialogType(type?: DialogType): SweetAlertType {

  if (type === DialogType.success) return 'success'

  if (type === DialogType.info) return 'info'

  if (type === DialogType.error) return 'error'

  if (type === DialogType.warning) return 'warning'

  if (type === DialogType.question) return 'question'

  return 'info'
}

export function infoDialog(title: string, message: string, langId: KnownLangs): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.info),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: getI18n(langId,'Ok'),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.debug("dialogHelper", message)

  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}

/**
 * this automatically logs the message
 * @param title
 * @param message
 * @param langId
 */
export function errorDialog(title: string, message: string, langId: KnownLangs): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.error),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: getI18n(langId,'Ok'),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.error(message)

  //this sometimes displays more error messages than we want...
  //error handling is not properly implemented... e.g. if loading tags failed then abort and not try to load all closed exercises
  // if (swal.getQueueStep() !== null) {
  //   swal.insertQueueStep(options)
  // }
  // else {
  //   swal.queue([options])
  // }

  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}


export function warningDialog(title: string, message: string, langId: KnownLangs): Promise<void> {
  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.warning),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: false,
    confirmButtonText: getI18n(langId,'Ok'),
    allowOutsideClick: false,
    allowEnterKey: true,
  }

  Logger.warn(message)

  // if (swal.getQueueStep() !== null) {
  //   swal.insertQueueStep(options)
  // }
  // else {
  //   swal.queue([options])
  // }



  return new Promise<void>((resolve) => {

    swal(options)
      .then(() => {
        resolve()
      })
  })
}

export function askDialog(title: string, message: string, langId: KnownLangs): Promise<boolean> {

  const options: SweetAlertOptions = {
    title: title,
    text: message,
    type: translateDialogType(DialogType.question),
    showConfirmButton: true,
    showCloseButton: false,
    showCancelButton: true,
    confirmButtonText: getI18n(langId,'Yes'),
    cancelButtonText: getI18n(langId,'No'),
    allowOutsideClick: false,
    allowEnterKey: true,
    focusCancel: true,
  }

  return new Promise<boolean>((resolve) => {

    Logger.debug("dialogHelper", `showing dialog title ${title}...`)

    swal(options)
      .then(() => {
        Logger.debug("dialogHelper", `...dialog title ${title} user took yes`)
        resolve(true)
      }, () => {
        Logger.debug("dialogHelper", `...dialog title ${title} user took no`)
        resolve(false)
      })
  })


}
